@@ -13,6 +13,7 @@ from ase.visualize.plot import plot_atoms
 from pymatgen.io.ase import AseAtomsAdaptor
 import copy
 import time
+import os
 
 
 class Dos:
@@ -22,20 +23,23 @@ class Dos:
     Parameters:
         folder (str): This is the folder that contains the VASP files.
         spin (str): Which spin direction to parse ('up' or 'down')
+        combination_method (str): If the spin option is 'both', the combination method can either be additive or substractive
+            by passing 'add' or 'sub'. It spin is passed as 'up' or 'down' this option is ignored.
     """
 
-    def __init__(self, folder, spin='up'):
+    def __init__(self, folder, spin='up', combination_method="add"):
         self.folder = folder
         self.spin = spin
+        self.combination_method = combination_method
         self.forbitals = False
         self.vasprun = Vasprun(
-            f'{folder}/vasprun.xml',
+            os.path.join(folder, 'vasprun.xml'),
             parse_dos=True,
             parse_eigen=False,
             parse_potcar_file=False
         )
         self.poscar = Poscar.from_file(
-            f'{folder}/POSCAR',
+            os.path.join(folder, 'POSCAR'),
             check_for_POTCAR=False,
             read_velocities=False
         )
@@ -101,10 +105,16 @@ class Dos:
                 'density': spin_factor * np.array(tdos.densities[self.spin_dict[self.spin]])
             }
         elif self.spin == 'both':
-            tdos_dict = {
-                'energy': np.array(tdos.energies - tdos.efermi),
-                'density': np.array(tdos.densities[Spin.up]) + np.array(tdos.densities[Spin.down])
-            }
+            if self.combination_method == "add":
+                tdos_dict = {
+                    'energy': np.array(tdos.energies - tdos.efermi),
+                    'density': np.array(tdos.densities[Spin.up]) + np.array(tdos.densities[Spin.down])
+                }
+            if self.combination_method == "sub":
+                tdos_dict = {
+                    'energy': np.array(tdos.energies - tdos.efermi),
+                    'density': np.array(tdos.densities[Spin.up]) - np.array(tdos.densities[Spin.down])
+                }
 
         return tdos_dict
 
@@ -241,9 +251,7 @@ class Dos:
         element_dict = {element: [] for element in elements}
 
         for element in elements:
-            element_index = np.where(np.isin(element, element_list))[0]
-            print(element_list)
-            print(element_index)
+            element_index = np.where(np.isin(element_list, element))[0]
             df = pd.concat(
                 [self.pdos_dict[i] for i in element_index],
                 axis=1
@@ -301,11 +309,60 @@ class Dos:
 
         return spd_df
 
-    def _set_density_lims(self, ax, tdensity, tenergy, erange, energyaxis, spin):
+    def _set_density_lims(self, ax, tdensity, tenergy, erange, energyaxis, spin, partial=False, is_dict=False, idx=None, multiple=False):
         energy_in_plot_index = np.where(
             (tenergy > erange[0]) & (tenergy < erange[1])
         )[0]
-        density_in_plot = tdensity[energy_in_plot_index]
+
+        if partial:
+            if is_dict:
+                if multiple:
+                    total = []
+                    for i in idx:
+                        first = i[0]
+                        second = i[1]
+                        total.append(tdensity[first][second][energy_in_plot_index])
+
+                    if spin == 'up' or spin == 'both':
+                        density_in_plot = total[np.argmax(np.max(total, axis=1))]
+                    elif spin == 'down':
+                        density_in_plot = total[np.argmin(np.min(total, axis=1))]
+                else:
+                    columns = []
+                    values = []
+                    dfs = []
+                    for i in idx:
+                        selected_densities = tdensity[i].iloc[energy_in_plot_index]
+                        dfs.append(selected_densities)
+                        if spin == 'up' or spin == 'both':
+                            max_value_column = selected_densities.max().idxmax() 
+                            max_value = selected_densities[max_value_column].max().max()
+                            columns.append(max_value_column)
+                            values.append(max_value)
+                        elif spin == 'down':
+                            min_value_column = selected_densities.min().idxmin() 
+                            min_value = selected_densities[min_value_column].min().min()
+                            columns.append(min_value_column)
+                            values.append(min_value)
+
+                    if spin=='up' or spin=='both':
+                        max_column = columns[np.argmax(values)]
+                        max_df = dfs[np.argmax(values)]
+                        density_in_plot = max_df[max_column].__array__()
+                    elif spin=='down':
+                        min_column = columns[np.argmin(values)]
+                        min_df = dfs[np.argmin(values)]
+                        density_in_plot = min_df[min_column].__array__()
+
+            else:
+                selected_densities = tdensity.iloc[energy_in_plot_index]
+                if spin=='up' or spin=='both':
+                    density_in_plot = selected_densities[selected_densities.max().idxmax()].__array__()
+                elif spin=='down':
+                    density_in_plot = selected_densities[selected_densities.min().idxmax()].__array__()
+
+        else:
+            density_in_plot = tdensity[energy_in_plot_index]
 
         if len(ax.lines) == 0:
             if energyaxis == 'y':
@@ -493,6 +550,16 @@ class Dos:
                 energyaxis=energyaxis,
                 erange=erange,
             )
+        else:
+            self._set_density_lims(
+                ax=ax,
+                tdensity=spd_df,
+                tenergy=tdos_dict['energy'],
+                erange=erange,
+                energyaxis=energyaxis,
+                spin=self.spin,
+                partial=True,
+            )
 
         for orbital in order:
             if sigma > 0:
@@ -617,6 +684,19 @@ class Dos:
                 energyaxis=energyaxis,
                 erange=erange,
             )
+        else:
+            self._set_density_lims(
+                ax=ax,
+                tdensity=self.pdos_dict,
+                tenergy=tdos_dict['energy'],
+                erange=erange,
+                energyaxis=energyaxis,
+                spin=self.spin,
+                partial=True,
+                is_dict=True,
+                idx=atom_orbital_pairs,
+                multiple=True,
+            )
 
         for i, atom_orbital_pair in enumerate(atom_orbital_pairs):
             atom = atom_orbital_pair[0]
@@ -747,6 +827,16 @@ class Dos:
                 energyaxis=energyaxis,
                 erange=erange,
             )
+        else:
+            self._set_density_lims(
+                ax=ax,
+                tdensity=orbital_df,
+                tenergy=tdos_dict['energy'],
+                erange=erange,
+                energyaxis=energyaxis,
+                spin=self.spin,
+                partial=True,
+            )
 
         for i, orbital in enumerate(orbitals):
             if sigma > 0:
@@ -871,6 +961,16 @@ class Dos:
                 energyaxis=energyaxis,
                 erange=erange,
             )
+        else:
+            self._set_density_lims(
+                ax=ax,
+                tdensity=atom_df,
+                tenergy=tdos_dict['energy'],
+                erange=erange,
+                energyaxis=energyaxis,
+                spin=self.spin,
+                partial=True,
+            )
 
         for i, atom in enumerate(atoms):
             if sigma > 0:
@@ -998,6 +1098,16 @@ class Dos:
                 energyaxis=energyaxis,
                 erange=erange,
             )
+        else:
+            self._set_density_lims(
+                ax=ax,
+                tdensity=pd.DataFrame(element_dict),
+                tenergy=tdos_dict['energy'],
+                erange=erange,
+                energyaxis=energyaxis,
+                spin=self.spin,
+                partial=True,
+            )
 
         for i, element in enumerate(elements):
             if sigma > 0:
@@ -1128,6 +1238,19 @@ class Dos:
                 energyaxis=energyaxis,
                 erange=erange,
             )
+        else:
+            self._set_density_lims(
+                ax=ax,
+                tdensity=element_dict,
+                tenergy=tdos_dict['energy'],
+                erange=erange,
+                energyaxis=energyaxis,
+                spin=self.spin,
+                partial=True,
+                is_dict=True,
+                idx=element_orbital_pairs,
+                multiple=True,
+            )
 
         for (i, element_orbital_pair) in enumerate(element_orbital_pairs):
             element = element_orbital_pair[0]
@@ -1269,6 +1392,18 @@ class Dos:
                 energyaxis=energyaxis,
                 erange=erange,
             )
+        else:
+            self._set_density_lims(
+                ax=ax,
+                tdensity=element_dict,
+                tenergy=tdos_dict['energy'],
+                erange=erange,
+                energyaxis=energyaxis,
+                spin=self.spin,
+                partial=True,
+                is_dict=True,
+                idx=elements,
+            )
 
         for element in elements:
             for i, orbital in enumerate(order):
